
/* Chrysalide - Outil d'analyse de fichiers binaires
 * operand.c - gestion générique des opérandes
 *
 * Copyright (C) 2008-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "operand.h"


#include <malloc.h>
#include <string.h>


#include "operand-int.h"



/* Initialise la classe générique des opérandes. */
static void g_arch_operand_class_init(GArchOperandClass *);

/* Initialise une instance d'opérande d'architecture. */
static void g_arch_operand_init(GArchOperand *);

/* Supprime toutes les références externes. */
static void g_arch_operand_dispose(GArchOperand *);

/* Procède à la libération totale de la mémoire. */
static void g_arch_operand_finalize(GArchOperand *);



/* Indique le type défini pour un opérande d'architecture. */
G_DEFINE_TYPE(GArchOperand, g_arch_operand, G_TYPE_OBJECT);



/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe générique des opérandes.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_arch_operand_class_init(GArchOperandClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_arch_operand_dispose;
    object->finalize = (GObjectFinalizeFunc)g_arch_operand_finalize;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise une instance d'opérande d'architecture.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_arch_operand_init(GArchOperand *operand)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_arch_operand_dispose(GArchOperand *operand)
{
    G_OBJECT_CLASS(g_arch_operand_parent_class)->dispose(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_arch_operand_finalize(GArchOperand *operand)
{
	g_arch_operand_set_alt_text(operand, NULL, RTT_COUNT);

    G_OBJECT_CLASS(g_arch_operand_parent_class)->finalize(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : a = premier opérande à consulter.                            *
*                b = second opérande à consulter.                             *
*                                                                             *
*  Description : Compare un opérande avec un autre.                           *
*                                                                             *
*  Retour      : Bilan de la comparaison.                                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_arch_operand_compare(const GArchOperand *a, const GArchOperand *b)
{
    bool result;                            /* Bilan à faire remonter      */

    result = (G_OBJECT_TYPE(G_OBJECT(a)) == G_OBJECT_TYPE(G_OBJECT(b)));

    if (result)
        result = G_ARCH_OPERAND_GET_CLASS(a)->compare(a, b);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                text    = représentation lisible alternative.                *
*                tag     = style d'impression pour le remplacement.           *
*                                                                             *
*  Description : Définit une autre représentation textuelle pour l'opérande.  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_arch_operand_set_alt_text(GArchOperand *operand, const char *text, RenderingTagType tag)
{
    if (operand->alt_text != NULL)
        free(operand->alt_text);

    if (text != NULL)
    {
        operand->alt_text = strdup(text);
        operand->alt_len = strlen(text);
        operand->alt_tag = tag;
    }
    else
    {
        operand->alt_text = NULL;
        operand->alt_len = 0;
        operand->alt_tag = RTT_COUNT;
    }

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                line    = ligne tampon où imprimer l'opérande donné.         *
*                syntax  = type de représentation demandée.                   *
*                                                                             *
*  Description : Traduit un opérande en version humainement lisible.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_arch_operand_print(const GArchOperand *operand, GBufferLine *line, AsmSyntax syntax)
{
    if (operand->alt_text != NULL)
        g_buffer_line_append_text(line, BLC_ASSEMBLY,
                                  operand->alt_text,
                                  operand->alt_len,
                                  operand->alt_tag,
                                  NULL);

    else
        G_ARCH_OPERAND_GET_CLASS(operand)->print(operand, line, syntax);

}
