
/* Chrysalide - Outil d'analyse de fichiers binaires
 * register.c - opérandes visant un registre x86
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "register.h"


#include "../../operand-int.h"



/* Définition d'un opérande visant un registre x86 (instance) */
struct _GX86RegisterOperand
{
    GArchOperand parent;                    /* Instance parente            */

    GX86Register *reg;                      /* Registre représenté         */

};


/* Définition d'un opérande visant un registre x86 (classe) */
struct _GX86RegisterOperandClass
{
    GArchOperandClass parent;               /* Classe parente              */

};


/* Initialise la classe des opérandes de registre x86. */
static void g_x86_register_operand_class_init(GX86RegisterOperandClass *);

/* Initialise une instance d'opérande de registre x86. */
static void g_x86_register_operand_init(GX86RegisterOperand *);

/* Compare un opérande avec un autre. */
static bool g_x86_register_operand_compare(const GX86RegisterOperand *, const GX86RegisterOperand *);

/* Traduit un opérande en version humainement lisible. */
static void g_x86_register_operand_print(const GX86RegisterOperand *, GBufferLine *, AsmSyntax);



/* Indique le type défini par la GLib pour un opérande de registre x86. */
G_DEFINE_TYPE(GX86RegisterOperand, g_x86_register_operand, G_TYPE_ARCH_OPERAND);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des opérandes de registre x86.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_register_operand_class_init(GX86RegisterOperandClass *klass)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise une instance d'opérande de registre x86.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_register_operand_init(GX86RegisterOperand *operand)
{
    GArchOperand *parent;                   /* Instance parente            */

    parent = G_ARCH_OPERAND(operand);

    parent->compare = (operand_compare_fc)g_x86_register_operand_compare;
    parent->print = (operand_print_fc)g_x86_register_operand_print;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data = flux de données à analyser.                           *
*                pos  = position courante dans ce flux. [OUT]                 *
*                len  = taille totale des données à analyser.                 *
*                size = taille de l'opérande, et donc du registre.            *
*                base = indice du premier registre.                           *
*                                                                             *
*  Description : Crée un opérande visant un registre x86.                     *
*                                                                             *
*  Retour      : Opérande mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_x86_register_operand_new_from_opcode(const bin_t *data, off_t *pos, off_t len, MemoryDataSize size, bin_t base)
{
    GX86RegisterOperand *result;            /* Structure à retourner       */
    GX86Register *reg;                      /* Registre lu                 */

    reg = g_x86_register_new(size, data[*pos] - base);

    if (reg != NULL)
    {
        (*pos)++;

        result = g_object_new(G_TYPE_X86_REGISTER_OPERAND, NULL);

        result->reg = reg;

    }
    else result = NULL;

    return G_ARCH_OPERAND(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data  = flux de données à analyser.                          *
*                pos   = position courante dans ce flux. [OUT]                *
*                len   = taille totale des données à analyser.                *
*                size  = taille de l'opérande, et donc du registre.           *
*                first = indique la partie du ModR/M à traiter.               *
*                                                                             *
*  Description : Crée un opérande visant un registre x86.                     *
*                                                                             *
*  Retour      : Opérande mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_x86_register_operand_new_from_mod_rm(const bin_t *data, off_t *pos, off_t len, MemoryDataSize size, bool first)
{
    GX86RegisterOperand *result;            /* Structure à retourner       */
    bin_t index;                            /* Registre lu                 */
    GX86Register *reg;                      /* Registre créé               */

    if (first) index = data[*pos] & 0x07;
    else index = (data[*pos] & 0x38) >> 3;

    reg = g_x86_register_new(size, index);

    if (reg != NULL)
    {
        (*pos)++;

        result = g_object_new(G_TYPE_X86_REGISTER_OPERAND, NULL);

        result->reg = reg;

    }
    else result = NULL;

    return G_ARCH_OPERAND(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : index = indice du registre visé.                             *
*                size  = taille de l'opérande, et donc du registre.           *
*                                                                             *
*  Description : Crée un opérande visant un registre x86 donné.               *
*                                                                             *
*  Retour      : Opérande mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_x86_register_operand_new_from_index(bin_t index, MemoryDataSize size)
{
    GX86RegisterOperand *result;            /* Structure à retourner       */
    GX86Register *reg;                      /* Registre lu                 */

    reg = g_x86_register_new(size, index);

    if (reg != NULL)
    {
        result = g_object_new(G_TYPE_X86_REGISTER_OPERAND, NULL);

        result->reg = reg;

    }
    else result = NULL;

    return G_ARCH_OPERAND(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : a = premier opérande à consulter.                            *
*                b = second opérande à consulter.                             *
*                                                                             *
*  Description : Compare un opérande avec un autre.                           *
*                                                                             *
*  Retour      : Bilan de la comparaison.                                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool g_x86_register_operand_compare(const GX86RegisterOperand *a, const GX86RegisterOperand *b)
{
    return g_x86_register_compare(a->reg, b->reg);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                line    = ligne tampon où imprimer l'opérande donné.         *
*                syntax  = type de représentation demandée.                   *
*                                                                             *
*  Description : Traduit un opérande en version humainement lisible.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_register_operand_print(const GX86RegisterOperand *operand, GBufferLine *line, AsmSyntax syntax)
{
    g_x86_pool_operand_print(operand->reg, line, syntax);

}
